library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity video_decoder_intf is
  port (
    iport        : in std_logic_vector (7 downto 0);
    hport        : in std_logic_vector (7 downto 0);
    idq_in       : in std_logic;
    iclk_in      : in std_logic;
    ipgh         : in std_logic;
    ipgv         : in std_logic;
    data         : out std_logic_vector (15 downto 0); 
    waddr        : out std_logic_vector (10 downto 0);
    idq_out      : out std_logic;  
    iclk_out     : out std_logic;
    fil_level    : out std_logic_vector(3 downto 0);
    line_count   : out std_logic_vector(15 downto 0);
    frame_id     : out std_logic_vector(1 downto 0);
    pixel_counter: out std_logic_vector (10 downto 0);
    active_out   : out std_logic;
    reset        : in std_logic
  );
end video_decoder_intf;

architecture structural of video_decoder_intf is

signal active     : std_logic;
signal pix_count  : std_logic_vector (10 downto 0);
signal pixel_addr : std_logic_vector(10 downto 0);

signal line_counter : std_logic_vector(15 downto 0);
signal frame_counter : std_logic_vector(1 downto 0);

-- The following signals indicate how much of the
-- line was already written into  the  block  RAM
signal one_fourth     : std_logic;
signal half_line      : std_logic;
signal three_quarters : std_logic;
signal entire_line    : std_logic;
signal filling_level  : std_logic_vector(3 downto 0);

-- Comment the following line
signal pixel_data : std_logic_vector(15 downto 0);                      ----------------------------
signal line_start : std_logic_vector(10 downto 0) := "00000000000";     ----------------------------Test
                                                                        ----------------------------data
                                                                        ----------------------------remove
                                                                        ----------------------------from file 

begin

-- pixel address - where to store valid pixels in the block RAMs
process (iclk_in, reset)
begin
 if reset='1' then
    pixel_addr <= "00000000000";
 elsif iclk_in'event and iclk_in='1' then
   if ipgh='0' then
     pixel_addr <= "00000000000";
   elsif idq_in = '1' and active = '1' then
     pixel_addr <= pixel_addr + 1;
   end if;
 end if;
end process;

-- count the actual data coming from iport and hport.
-- Some data is control (FF, 00 , 00 , SAV business)
-- Reset the counter whenever ipgh is zero
process (iclk_in, reset)
begin
  if reset='1' then
    pix_count <= "00000000000";
  elsif iclk_in'event and iclk_in='1' then
    if idq_in='1' then
      if ipgh='0' then
        pix_count <= "00000000000";
      else
        pix_count <= pix_count + 1;
      end if; 
    end if;
  end if;
end process;

-- count the number of lines
process (iclk_in, reset)
begin
  if reset='1' then
    line_counter <= X"0000";
  elsif iclk_in'event and iclk_in='1' then
    if ipgv='0' then
      line_counter <= X"0000";
    elsif ipgh='1' and pix_count=719 then
      line_counter <= line_counter + 1;
    end if;
  end if;
end process;

-- give the frame ID
process (iclk_in, reset)
begin
  if reset='1' then
    frame_counter <= "00";
  elsif iclk_in'event and iclk_in='1' then
    if line_counter = 239 and pix_count=719 then
      frame_counter <= frame_counter+1;
    end if;
  end if;
end process;

-- Active means we are within
-- the horizontal line active video
process (iclk_in)
begin
 if iclk_in'event and iclk_in='1' then
   if ipgh='0' then
     active <= '0';
   elsif pix_count = 1 then
     active <= '1';
   elsif pix_count=720 then
     active <= '0';
   end if;
 end if;
end process;

-- Set output signals according to where
-- in the current line the video decoder
-- is writing the block RAM
process (iclk_in, reset)
begin
 if reset='1' then
   one_fourth <= '0';
 elsif iclk_in'event and iclk_in='1' then
   if pix_count=0 then
     one_fourth <= '0';
   elsif pix_count=161 then
     one_fourth <= '1';
   end if;
 end if;
end process;

process (iclk_in, reset)
begin
 if reset='1' then
   half_line <= '0';
 elsif iclk_in'event and iclk_in='1' then
   if pix_count=0 then
     half_line <= '0';
   elsif pix_count=321 then
     half_line <= '1';
   end if;
 end if;
end process;

process (iclk_in, reset)
begin
 if reset='1' then
   three_quarters <= '0';
 elsif iclk_in'event and iclk_in='1' then
   if pix_count=0 then
     three_quarters <= '0';
   elsif pix_count=481 then
     three_quarters <= '1';
   end if;
 end if;
end process;

process (iclk_in, reset)
begin
 if reset='1' then
   entire_line <= '0';
 elsif iclk_in'event and iclk_in='1' then
   if pix_count=0 then
     entire_line <= '0';
   elsif pix_count=641 then
     entire_line <= '1';
   end if;
 end if;
end process;


filling_level(0) <= one_fourth;
filling_level(1) <= half_line;
filling_level(2) <= three_quarters;
filling_level(3) <= entire_line;


-- Output signals of this entity

fil_level <= filling_level;
line_count <= line_counter;
frame_id <= frame_counter;

  data(15 downto 8) <= iport;-----------------------------------------these have to be reinserted
  data(7 downto 0)  <= hport;-----------------------------------------these have to be reinserted
  idq_out <= idq_in;         -----------------------------------------these have to be reinserted


waddr <= pixel_addr;
iclk_out <= iclk_in;
pixel_counter <= pix_count;
active_out <= active;



 
-- -- Test generator
-- data <= pixel_data;
-- idq_out <= '1';
-- line_start <= pix_count;  -- - line_counter - 4;
--  -- begin
--    --This is the test date for the line finder - we want to insert and remove
--    --this one as much as possible
--  -- pixel data
--  process (iclk_in, reset)
--  begin
--    if reset='1' then
--      pixel_data <= X"0000";
--    elsif iclk_in'event and iclk_in = '1' then
--      if line_start=12 then
--        pixel_data <= X"0000";
--      elsif line_start=45 then
--        pixel_data <= X"F000";
--      elsif line_start=100 then
--        pixel_data <= X"0000";
--      elsif line_start=200 then
--        pixel_data <= X"F000";
--      elsif line_start=225 then
--        pixel_data <= X"0000";
--      elsif line_start=275 then
--        pixel_data <= X"F000";
--      elsif line_start=306 then
--        pixel_data <= X"0000";
--       elsif line_start=382 then
--         pixel_data <= X"F000";
--       elsif line_start=383 then
--         pixel_data <= X"0000";
--      elsif line_start=456 then
--        pixel_data <= X"F000";              
--      end if;
--    end if;
--  end process;

-- --ends here the test data


-- -- pixel address - where to store in the block RAMs
-- process (iclk_in, reset)
-- begin
--   if reset='1' then
--      pixel_addr <= "00000000000";
--   elsif iclk_in'event and iclk_in='1' then
--      pixel_addr <= pixel_addr + 1;
--   end if;
-- end process;


-- data <= pixel_data;
-- waddr <= pixel_addr;
-- iclk_out <= iclk_in;
-- idq_out <= '1';

 end structural;

